require ('emoticoner.rb')


class ChatController < ApplicationController
  protect_from_forgery :only => [:index]
  before_filter :authorize, :except => [:juggernaut_connection_logout, :juggernaut_subscription]
  before_filter :check_room, :only => [:rooms, :send_data]

  def index
    @rooms = ChatRoom.find(:all)
    render :layout => 'chat'
  end

  def rooms
    render :layout => 'rooms'
  end

  def send_data
    # Stosujemy h ponieważ userzy nie mogą przesyłać niczego poza tekstem
    msg = Emoticoner.emoticate(h(params[:chat_input]))

    render :juggernaut => {:channels => [@room.id], :type => :send_to_channels} do |page|
      date = "[#{Time.now.strftime("%d-%m-%y %H:%M")}] "
      nick = "<div id=\"nick\" style=\"color: #{session[:color]}\">#{@user.login.capitalize}</div>"
      page.insert_html :bottom, 'chat_data', date+nick+': '+msg+"<br/>"
      page << 'var objDiv = document.getElementById("chat_data"); objDiv.scrollTop = objDiv.scrollHeight;'
      page << "DHTMLSound('/sounds/notice.wav');"
    end
    render :nothing => true
  end

  def juggernaut_connection_logout
    room = ChatRoom.find(params[:channels].first.to_i)
    user = User.find(params[:client_id])
    ChatRoomsUser.disconnect(user, room)
    users_list = ''
    room.users.each { |u| users_list+= "<li>#{u.login.capitalize}</li>" }
    render :juggernaut => {:channels => [params[:channels].first.to_i], :type => :send_to_channels} do |page|
      page.replace_html 'users_list', users_list
    end

    render :nothing => true, :status => 200
  end

  def juggernaut_subscription
    room = ChatRoom.find(params[:channels].first.to_i)
    user = User.find(params[:client_id])

    ChatRoomsUser.connect(user, room)

    users_list = ''
    room.users.each { |u| users_list+= "<li>#{u.login.capitalize}</li>" }
    render :juggernaut => {:channels => [params[:channels].first.to_i], :type => :send_to_channels} do |page|
      page.replace_html 'users_list', users_list
    end

    render :nothing => true, :status => 200

  end

  private

  def check_room
    @room = ChatRoom.find_by_link(params[:id])
    if @room.nil?
      flash[:error] = "Wybrany pokój nie istnieje"
      redirect_to :action => :index
    end
  end

end
