# Potrzebne nam do hashowania hasła
require 'digest/sha2'

class User < ActiveRecord::Base

  LOGIN_MAX_LENGTH    = 14
  LOGIN_MIN_LENGTH    = 5
  PASSWORD_MAX_LENGTH = 16
  PASSWORD_MIN_LENGTH = 8

  attr_accessor :password_confirmation
  attr_accessor :password
  attr_accessor :current_password

  validates_uniqueness_of :login, :message => 'który wybrałeś jest już zajęŧy'
  validates_length_of :login, :within =>  LOGIN_MIN_LENGTH..LOGIN_MAX_LENGTH,
    :too_long => "jest za długi",
    :too_short => "jest za krótki"
  validates_format_of :login, :with => /^[A-ZąćęłńóśżźĄĆĘŁŃÓŚŹŻ0-9_]*$/i,
    :message => 'jest nieprawidłowy'
  validates_format_of :password, :with => /^[A-ZąćęłńóśżźĄĆĘŁŃÓŚŹŻ0-9_]*$/i,
    :message => 'musi składać się z liter oraz cyfr'
  validates_length_of :password, :within => PASSWORD_MIN_LENGTH..PASSWORD_MAX_LENGTH,
    :too_long => "jest za długie",
    :too_short => "jest za krótkie"
  validates_inclusion_of :account, :in => ["admin", "user"],
    :message => 'Wybierz prawidłowy typ konta (user lub admin)'
  validates_confirmation_of :password,
    :message => "musi zgadzać się z powtórzeniem"

  HUMANIZED_ATTRIBUTES = {
    :password => 'Hasło'
  }

  def self.human_attribute_name(attr)
    HUMANIZED_ATTRIBUTES[attr.to_sym] || super
  end

  # Sprawdza czy user o podanym loginie i hasle istnieje
  # Jesli tak to go zwraca
  def self.authenticate(login, password)
    user = self.find_by_login(login)
    if user
      expected_password = encrypted_password(password, user.salt)
      if user.hashed_password != expected_password
        user = nil
      end
    end
    user
  end

  def password
    @password
  end

  # Zapis hasła zahashowanego i posolonego - jesli puste to nie zmieni hasła
  def password=(pwd)
    @password = pwd
    return if pwd.blank?
    create_new_salt
    self.hashed_password = User.encrypted_password(self.password, self.salt)
  end

  # Loguje usera do systemu
  def login!(session)
    session[:user_id] = self.id
  end

  # Wylogowuje czy sie jest userem czy nie
  def self.logout!(session)
    session[:user_id] = nil
  end

  # Usuwa haslo zeby nie wyswietlalo go spowrotem w formularzu
  # Bo skoro pokazalo znowu formularz rejestr/logowania tzn ze cos z haslem
  # bylo nie tak wiec po co wyswietlac znowu złe hasło?
  def clear_password!
    self.password = nil
    self.password_confirmation = nil
  end


  def account_type
    return :user if self.account == 'user'
    return :admin if self.account == 'admin'
  end

  protected

  # Tworzy nową sól do hasła
  def create_new_salt
    self.salt = self.object_id.to_s + rand.to_s
  end

  # Tworzy nowe zaszyfrowane hasło z podanego hasła oraz soli
  def self.encrypted_password(password, salt)
    return if password.nil? || salt.nil?
    string_to_hash = password + "GB609ru30j)(*&Th" + salt
    Digest::SHA2.hexdigest(string_to_hash)
  end


end
